--[[
	need to:
	
	Disable everything if the character is not level 20.
	Trigger scanning at the start of a BG, instead of by slash command.
	Decide when data is to be shared (start of a bg would be good, or have characters declare when they go AFK so other AFK chars can send then).
	When merging data, update the first seen date if it's earlier than the one stored.
	
	Store addon version in savedSettings, to avoid merging incompatible databases.
	Allow for database sections for different character levels, so other brackets may use the addon.
--]]

local equipmentSlots = {
	"HeadSlot",
	"NeckSlot",
	"ShoulderSlot",
	"ChestSlot",
	"WaistSlot",
	"LegsSlot",
	"FeetSlot",
	"WristSlot",
	"HandsSlot",
	"Finger0Slot", --don't need to check both, as it's 'what can fit in this slot', not 'what's in this slot'.
	"Trinket0Slot", --ditto
	"BackSlot",
	"MainHandSlot",
	"SecondaryHandSlot", --unfortunately this means one-hand items end up showing twice because they can go in either hand.  Doesn't matter, just means they get sent to the DB twice, and there's an overwrite of the first.
}

local slotNames = {
	"Head",
	"Neck",
	"Shoulder",
	"Back",
	"Chest",
	"Wrist",
	"Hands",
	"Waist",
	"Legs",
	"Feet",
	"Finger",
	"Trinket",
	"Ranged",
	"Two-Hand",
	"One-Hand",
	"Main Hand",
	"Off Hand",
	"Held In Off-Hand",
}

local armorTypes = {
	"Cloth",
	"Leather",
	"Mail",
	"Plate",
}

local excludeItems = {
	"Fishing Pole", --excludes all fishing poles
	"Mining Pick",
	"Skinning Knife",
	"Burgy Blackheart's Handsome Hat", --instead of listing everything here, might be better to just exclude items with no stats (not counting armor)
}

local statMatchStrings = {
	["Armor"] = "^(%d+) Armor$",
	["Stamina"] = "^+(%d+) Stamina$",
	["Strength"] = "^+(%d+) Strength$",
	["Agility"] = "^+(%d+) Agility$",
	["Intellect"] = "^+(%d+) Intellect$",
	["Spirit"] = "^+(%d+) Spirit$",
	["Crit"] = "^+(%d+) Critical Strike$",
	["Haste"] = "^+(%d+) Haste$",
	["Dodge"] = "^+(%d+) Dodge$",
	["BonusArmor"] = "^+(%d+) Bonus Armor$",
	["Resilience"] = "^+(%d+) PVP Resilience$",
	["FireResist"] = "^+(%d+) Fire Resistance$",
	["ShadowResist"] = "^+(%d+) Shadow Resistance$",
	["SpellPower"] = "^+(%d+) Spell Power$",
	["AttackPower"] = "^+(%d+) Attack Power$",
	["Speed"] = "^Speed ([%.%d]+)$",
	["DPS"] = "^%(([%.%d]+) damage per second%)$",
	["DamageLow"] = "^(%d+)[%s%-%d]- Damage$",
	["DamageHigh"] = "^[%s%-%d]-(%d+) Damage$",
	["ItemID"] = "^itemID=(%d+)$", --not an official one, but we append it onto data pulled from the tooltip, and use this to format it for output
	--on use
	--on equip
	--chance on hit
}



local equippableItems = {}
local equipmentIn = {}
local equipmentProcessing = {}



if not EquipmentScale_Variables then
	EquipmentScale_Variables = {}
end



-- create the tootip frame that we'll be reading data from
local tip = CreateFrame("GameTooltip", "ScaledEquipment")
local tooltipLines = {}
local line = 0
for i = 1, 20 do
	local left, right = tip:CreateFontString(), tip:CreateFontString()
	left:SetFontObject(GameFontNormal)
	right:SetFontObject(GameFontNormal)

	tip:AddFontStrings(left, right)

	line = line + 1
	tooltipLines[line] = left

	line = line + 1
	tooltipLines[line] = right
end
tip:SetOwner(UIParent, "ANCHOR_NONE") --attach it to the main UI frame, but give it no location so it isn't visible



--hash function to provide different table indices for the same items with different stats, should bliz change things
local function StringHash(text)
	local counter = 1
  local len = string.len(text)
  for i = 1, len, 3 do 
    counter = math.fmod(counter*8161, 4294967279) +  -- 2^32 - 17: Prime!
  	  (string.byte(text,i)*16776193) +
  	  ((string.byte(text,i+1) or (len-i+256))*8372226) +
  	  ((string.byte(text,i+2) or (len-i+256))*3932164)
  end
  return math.fmod(counter, 4294967291) -- 2^32 - 5: Prime (and different from the prime in the loop)
end



--sorts the item data table before converting it into a string, so that hashing will be consistent, given that Lua key=value tables aren't consistently ordered
local function tableToString(tbl)
	
	local str = ""
	local orderedTable = {}
	
	for k, v in pairs(tbl) do
		table.insert(orderedTable, {key=k,value=v})
	end
	table.sort(orderedTable,function(a, b) return a.key < b.key end)
	for i, v in ipairs (orderedTable) do
		str = str..v.key.."="..v.value.."::"
	end
	return str
end



--format the data and save it out to savedSettings
local function saveEquipment()
	for i, v in pairs(equipmentIn) do
		equipmentProcessing = {}
		equipmentProcessing["Name"] = equipmentIn[i][1]
		for j,w in pairs(equipmentIn[i]) do
			for statName, matchString in pairs(statMatchStrings) do
				if string.match(w, matchString) then
					equipmentProcessing[statName] = string.match(w, matchString)
				end
			end
			if tContains(armorTypes,w) then
				equipmentProcessing["ArmorType"] = w
			elseif tContains(slotNames,w) then
				equipmentProcessing["Slot"] = w
			end
		end
		local hash = StringHash(tableToString(equipmentProcessing))
		EquipmentScale_Variables[hash] = {}
		EquipmentScale_Variables[hash]["Item"] = equipmentProcessing
		EquipmentScale_Variables[hash]["LastSeenOnDate"] = date("%y%m%d")
		if not EquipmentScale_Variables[hash]["FirstSeenOnDate"] then
			EquipmentScale_Variables[hash]["FirstSeenOnDate"] = date("%y%m%d")
		end
	end
end



--dump all equippable items into equipmentIn{}
local function scanEquipped()
	local inTableIndex = 0
	for i, v in pairs(equipmentSlots) do
		local id, texture, checkRelic = GetInventorySlotInfo(v)
		equippableItems = {} --clear the table
		GetInventoryItemsForSlot(id, equippableItems) --function populates the table equippableItems{} only with the items for slot 'id' that can be equipped on this character, and are currently in your inventory.  For some reason this includes crafted items, that require a skill level, even if you don't have the skill.
		for locationBitstring, itemID in pairs(equippableItems) do
			local player, bank, bags, voidstorage, slot, bag = EquipmentManager_UnpackLocation(locationBitstring)
			tip:ClearLines()
			if player and (not bags) then
				tip:SetInventoryItem("player", slot)
			elseif player and bags then
				tip:SetBagItem(bag, slot)
			else
				break
			end
			equipmentIn[inTableIndex] = {}
			local excludeItem = false
			for c = 1, #tooltipLines do
				local text = tooltipLines[c]:GetText()  -- what GetText() returns doesn't seem to be a purely text string, and attempting to concatenate it with strings was causing WoW to crash to desktop.  Other functions are working fine though.
				if text then
					equipmentIn[inTableIndex][c]=text
					--set flag if it's an item in the above list, or an engineering item that can't be trained for on an f2p
					if tContains(excludeItems,text) or ( string.match(text, "^Requires Engineering %((%d+)%)$") and tonumber(string.match(text, "^Requires Engineering %((%d+)%)$")) > 100 ) then
						excludeItem = true
					end
				end
			end
			if not excludeItem then
				table.insert(equipmentIn[inTableIndex], "itemID="..itemID)
				inTableIndex = inTableIndex + 1
			else
				equipmentIn[inTableIndex] = nil
			end
		end
	end
	saveEquipment()
end



SLASH_ESCALE1 = "/es"
SlashCmdList["ESCALE"] = function(txt)
	scanEquipped()
end